/****************************************************************************
    INCLUDE FILES
****************************************************************************/

#include <dos.h>
#include <stdio.h>

/****************************************************************************
    TYPEDEFS AND DEFINES
****************************************************************************/

#define 	DEBUG 1

// change for hardware type

#define TS5300 1
#define TS3300 0


#define FALSE		0
#define TRUE		!FALSE

// register definitions for 12887 Real Time Clock

#define RTC_CMD_ADDR	0x70	// RTC internal register offset goes here
#define RTC_DAT_ADDR	0x71	// RTC internal register R/W access here

#define RTC_REG_A		0x0A	// RTC register offset for accessing Reg. A
#define RTC_REG_B		0x0B	// RTC register offset for accessing Reg. B
#define RTC_REG_C		0x0C	// RTC register offset for accessing Reg. C
#define RTC_REG_D		0x0D	// RTC register offset for accessing Reg. D

// register definitions for 8259 programmable interrupt controller

#define INT_0_CTRL_ADDR	0x20	// 8259 master (0) interrupt control register
#define INT_0_MASK_ADDR	0x21	// 8259 master (0) interrupt mask register
#define INT_1_CTRL_ADDR	0xA0	// 8259 slave  (1) interrupt control register
#define INT_1_MASK_ADDR	0xA1	// 8259 slave  (1) interrupt mask register   

#define END_OF_INT		0x20	// generic end of interrupt mask

/****************************************************************************
    GLOBAL VARIABLES
****************************************************************************/

#if DEBUG
void interrupt(*NewVector)();   // holds our vector address, for debug
#endif

void interrupt(*OldVector)();   // holds the previous vector for INT8

volatile unsigned char ucPort;  // temp storage for port I/O
volatile unsigned char ucCount; // generic counter storage

/****************************************************************************
    FUNCTION PROTOTYPES
****************************************************************************/

void interrupt myISR( void );

/****************************************************************************
	EXTERNAL DECLARATIONS
****************************************************************************/

/****************************************************************************
    Function: main
 Description: initialization, main loop, de-initialization
      Passed: none
     Returns: none
*****************************************************************************/

main()
{

    // IRQ5 calls the interrupt service routine installed at vector 0x0D

    //-------------------------------------------------------------------------
    // Setup interrupt vectors so that IRQ5 calls our interrupt service routine
    //-------------------------------------------------------------------------

    OldVector = getvect(0x0d);              // get old int. vector to re-install on exit

    disable();                              // disable interrupts for next call

    setvect(0x0d,myISR);                    // set int. vector for our routine

    enable();                               // enable system interrupts


    //-------------------------------------------------------------------------
    // setup interrupt controller to enable IRQ5 (interrupt 5 on  8259)
    //-------------------------------------------------------------------------

    disable();  // don't let interrupts happen during read/modify/write

    ucPort = inportb( INT_0_MASK_ADDR );    // read interrupt mask, 8259(1)

    ucPort &= 0xDF;                         // clear mask bit for IRQ5

    outportb( INT_0_MASK_ADDR, ucPort );    // write modified mask 8259(1)

    enable();                               // re-enable system interrupts

#if TS3300
	ucPort = inportb( 0x7D );
	outportb( 0x7D, ucPort |= 0x80 );       // set bit to setup DIO1.12 as IRQ5
#else										// on TS-3300

	ucPort = inportb( 0x7A );
	outportb( 0x7A, ucPort |= 0x40 );		// set bit to setup DIO1.12 as IRQ5
#endif										// on TS-5300

	asm	int 3;

#if DEBUG
    NewVector = getvect(0x0d);              // get int. vector for RTC (IRQ5)

    printf( "OldVector=(%p)\n", OldVector );
    printf( "NewVector=(%p)\n", NewVector );
#endif


    while( !kbhit() );


    //-------------------------------------------------------------------------
    // setup interrupt controller to disable IRQ8 (interrupt 5 on second 8259)
    //-------------------------------------------------------------------------

    disable(); // don't let interrupts happen during read/modify/write

    ucPort = inportb( INT_0_MASK_ADDR );    // read interrupt mask, 8259(1)

    ucPort |= 0x20;                         // clear mask bit for IRQ5

    outportb( INT_0_MASK_ADDR, ucPort );    // write modified mask 8259(1)

    enable();  // re-enable system interrupts

    //-------------------------------------------------------------------------
    // Setup interrupt vectors so that IRQ8 calls our interrupt service routine
    //-------------------------------------------------------------------------

    disable();                              // disable interrupts for next call

    setvect( 0x0d, OldVector );             // set RTC int. vector for our routine

    enable();                               // enable system interrupts

}

/****************************************************************************
     Function: myISR
 Description: interrupt service routine
        Passed: none
      Returns: none
*****************************************************************************/

void interrupt myISR( void )
{

    asm int 3;

    outportb( INT_0_CTRL_ADDR, END_OF_INT );
}
